VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Geodesy"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
    
'************************************************************************
'
'       Windsway Geodesy Library
'       Version 0.91
'
' The original Visual Basic code was:
'       mapstuff.bas  version 0.91 released 06/25/2002
'       geodesy.cls   version 0.91 released 06/25/2002
'
' The initial developer of the Visual Basic code is < Victor Fraenckel>
' (vfraenc1@nycap.rr.com).
'
' Portions created by < Victor Fraenckel> are
' Copyright (C) 2002 < Windsway Instrument Company>.
'
'
'       Obtained through:
'
'       Windsway Instrument Company
'
' You may get the latest version of this file by email from
' Victor Fraenckel <vfraenc1@nycap.rr.com >
'
' The contents of this file are used with permission, subject to
' the Mozilla Public License Version 1.1 (the "License")' you may
' not use this file except in compliance with the License. You may
' obtain a copy of the License at
' http://www.mozilla.org/MPL/MPL-1.1.html
'
' Software distributed under the License is distributed on an
' "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
' implied. See the License for the specific language governing
' rights and limitations under the License.
'
'************************************************************************
'************************************************************************
'  revision history
'   date        comment                                       who
'
' 06/03/02      began development                             vhf
' 06/23/02      added datum names with their corresponding    vhf
'               indexes into the ellipsoid array
'               changed the names of the array sizes
'               Choices => EllipseChoices
'               DChoices => DatumChoices
'               Added NAD27 to name of ellipsoid 5
'
'************************************************************************

'************************************************************************
'  module contains the following routines:
'
'   name        last modified date
'
'Get procedures
' eMajor            06/03/02
' eMinor            06/03/02
' eName             06/03/02
' Choices           06/03/02
' DChoices          06/03/02
' DatumName         06/23/02
' DatumNumber       06/23/02
'
'************************************************************************
    
Private Const EllipseChoices As Integer = 22
Private Const DatumChoices As Integer = 103

Private ellipseMajor(1 To EllipseChoices) As Double
Private ellipseMinor(1 To EllipseChoices) As Double
Private ellipseName(1 To EllipseChoices) As String
Private datumName(1 To DatumChoices) As String
Private datumNumber(1 To DatumChoices) As Integer


Property Get eMajor(ByVal index) As Double

  eMajor = ellipseMajor(index)
  
End Property

Property Get eMinor(ByVal index) As Double

  eMinor = ellipseMinor(index)
  
End Property

Property Get ename(ByVal index) As String

  ename = ellipseName(index)
  
End Property


Property Get Choices() As Integer

  Choices = EllipseChoices
  
End Property
Property Get DChoices() As Integer

  DChoices = DatumChoices
  
End Property
Property Get dName(ByVal index) As String

  dName = datumName(index)
  
End Property

Property Get dNumber(ByVal index) As Integer

  dNumber = datumNumber(index)
  
End Property

Private Sub Class_Initialize()

  'ellipsoid major axes in meters
  ellipseMajor(1) = 6377563.396       '1  airy
  ellipseMajor(2) = 6377340.189       '2  airy modified
  ellipseMajor(3) = 6378160#          '3  australian national
  ellipseMajor(4) = 6377397.155       '4  bessel 1841
  ellipseMajor(5) = 6378206.4         '5  clarke 1866 - NAD27
  ellipseMajor(6) = 6378249.145       '6  clarke 1880
  ellipseMajor(7) = 6377276.345       '7  everest
  ellipseMajor(8) = 6377304.063       '8  everest modified
  ellipseMajor(9) = 6378166#          '9  fischer 1960
  ellipseMajor(10) = 6378155#         '10  fischer 1960 modified
  ellipseMajor(11) = 6378150#         '11 fischer 1968
  ellipseMajor(12) = 6378160#         '12 geodetic reference system 1967
  ellipseMajor(13) = 6378137#         '13 geodetic reference system 1980
  ellipseMajor(14) = 6378200#         '14 helmert
  ellipseMajor(15) = 6378270#         '15 hough
  ellipseMajor(16) = 6378388#         '16 international
  ellipseMajor(17) = 6378245#         '17 krassovsky
  ellipseMajor(18) = 6378160#         '18 south american
  ellipseMajor(19) = 6378165#         '19 wgs 60
  ellipseMajor(20) = 6378145#         '20 wgs 66
  ellipseMajor(21) = 6378135#         '21 wgs 72
  ellipseMajor(22) = 6378137#         '22 wgs 84
  
  'ellipsoid minor axes in meters
  ellipseMinor(1) = 6356256.909       '1  airy
  ellipseMinor(2) = 6356036.205       '2  airy modified
  ellipseMinor(3) = 6356774.719       '3  australian national
  ellipseMinor(4) = 6356078.963       '4  bessel 1841
  ellipseMinor(5) = 6356583.8         '5  clarke 1866 - NAD27
  ellipseMinor(6) = 6356514.87        '6  clarke 1880
  ellipseMinor(7) = 6356075.413       '7  everest
  ellipseMinor(8) = 6356103.039       '8  everest modified
  ellipseMinor(9) = 6356784.284       '9  fischer 1960
  ellipseMinor(10) = 6356773.32       '10  fischer 1960 modified
  ellipseMinor(11) = 6356768.337      '11 fischer 1968
  ellipseMinor(12) = 6356774.516      '12 geodetic reference system 1967
  ellipseMinor(13) = 6356752.314      '13 geodetic reference system 1980
  ellipseMinor(14) = 6356818.17       '14 helmert
  ellipseMinor(15) = 6356794.343      '15 hough
  ellipseMinor(16) = 6356911.946      '16 international
  ellipseMinor(17) = 6356863.019      '17 krassovsky
  ellipseMinor(18) = 6356778.304      '18 south american
  ellipseMinor(19) = 6356783.287      '19 wgs 60
  ellipseMinor(20) = 6356759.769      '20 wgs 66
  ellipseMinor(21) = 6356750.52       '21 wgs 72
  ellipseMinor(22) = 6356750.321      '22 wgs 84
  
  'ellipsoid common names
  ellipseName(1) = "Airy"                     '1
  ellipseName(2) = "Airy Modified"            '2
  ellipseName(3) = "Australian"               '3
  ellipseName(4) = "Bessel 1841"              '4
  ellipseName(5) = "Clark 1866 - NAD27"       '5
  ellipseName(6) = "Clarke 1880"              '6
  ellipseName(7) = "Everest"                  '7
  ellipseName(8) = "Everest Modified"         '8
  ellipseName(9) = "Fischer 1960"             '9
  ellipseName(10) = "Fischer 1960 Modified"   '10
  ellipseName(11) = "Fischer 1968"            '11
  ellipseName(12) = "Geodetic Reference 1967" '12
  ellipseName(13) = "Geodetic Reference 1980" '13
  ellipseName(14) = "Helmert"                 '14
  ellipseName(15) = "Hough"                   '15
  ellipseName(16) = "International"           '16
  ellipseName(17) = "Krassovsky"              '17
  ellipseName(18) = "South American 1969"     '18
  ellipseName(19) = "WGS-60"                  '19
  ellipseName(20) = "WGS-66"                  '20
  ellipseName(21) = "WGS-72"                  '21
  ellipseName(22) = "WGS-84"                  '22
  
  datumName(1) = "Adindan"                      '1
  datumName(2) = "Afgooye"                      '2
  datumName(3) = "Ain El Abd 1970"              '3
  datumName(4) = "Anna 1 Astro"                 '4
  datumName(5) = "ARC 1950"                     '5
  datumName(6) = "ARC 1960"                     '6
  datumName(7) = "Ascension Island 1958"        '7
  datumName(8) = "Astro B4 Sorol Atoll"         '8
  datumName(9) = "Astro Beacon E"               '9
  datumName(10) = "Astro Dos 71/4"              '10
  datumName(11) = "Astronomic Station 1952"     '11
  datumName(12) = "Australian Geodetic 1966"    '12
  datumName(13) = "Australian Geodetic 1984"    '13
  datumName(14) = "Bellevue IGN"                '14
  datumName(15) = "Bermuda 1957"                '15
  datumName(16) = "Bogota Observatory"          '16
  datumName(17) = "Campo Inchauspe"             '17
  datumName(18) = "Canto Astro 1966"            '18
  datumName(19) = "Cape South Africa"           '19
  datumName(20) = "Cape Canaveral"              '20
  datumName(21) = "Carthage"                    '21
  datumName(22) = "Chatham 1971"                '22
  datumName(23) = "Chua Astro"                  '23
  datumName(24) = "Corrego Alegre"              '24
  datumName(25) = "Djakarta"                    '25
  datumName(26) = "Dos 1968"                    '26
  datumName(27) = "Easter Island 1967"          '27
  datumName(28) = "European 1950"               '28
  datumName(29) = "European 1979"               '29
  datumName(30) = "*Finland Hayford"            '30
  datumName(31) = "Gandajika Base"              '31
  datumName(32) = "Geodetic Datum 1949"         '32
  datumName(33) = "Guam 1963"                   '33
  datumName(34) = "Gux 1 Astro"                 '34
  datumName(35) = "Hjorsey 1955"                '35
  datumName(36) = "Hong Kong 1963"              '36
  datumName(37) = "Indian"                      '37
  datumName(38) = "*Indian Thailand"            '38
  datumName(39) = "Ireland 1965"                '39
  datumName(40) = "ISTS 073 Astro 1969"         '40
  datumName(41) = "Johnson Island 1961"         '41
  datumName(42) = "Kandawala"                   '42
  datumName(43) = "Kergulen Island"             '43
  datumName(44) = "Kertau 1948"                 '44
  datumName(45) = "L. C. 5 Astro"               '45
  datumName(46) = "Liberia 1964"                '46
  datumName(47) = "*Luzon Mindanao"             '47
  datumName(48) = "Luzon Phillipines"           '48
  datumName(49) = "Mahe 1971"                   '49
  datumName(50) = "Marco Astro"                 '50
  datumName(51) = "Massawa"                     '51
  datumName(52) = "Merchich"                    '52
  datumName(53) = "Midway Astro 1961"           '53
  datumName(54) = "Minna"                       '54
  datumName(55) = "NAD27 Alaska"                '55
  datumName(56) = "NAD27 Bahamas"               '56
  datumName(57) = "NAD27 Canada"                '57
  datumName(58) = "NAD27 Canal Zone"            '58
  datumName(59) = "NAD27 Caribbean"             '59
  datumName(60) = "NAD27 Central America"       '60
  datumName(62) = "NAD27 Conus"                 '61
  datumName(62) = "NAD27 Cuba"                  '62
  datumName(63) = "NAD27 Greenland"             '63
  datumName(64) = "NAD27 Mexico"                '64
  datumName(65) = "NAD27 San Salvador"          '65
  datumName(66) = "NAD83"                       '66
  datumName(67) = "*Nahrwn Masirah Island"      '67
  datumName(68) = "*Nahrwn Saudi Arabia"        '68
  datumName(69) = "*Nahrwn United Arab Emirates" '69
  datumName(70) = "*Naparima BWI"               '70
  datumName(71) = "Observatorio 1966"           '71
  datumName(72) = "Old Egyptian"                '72
  datumName(73) = "Old Hawaiian"                '73
  datumName(74) = "Oman"                        '74
  datumName(75) = "Ord Survey G. Britian 1936"  '75
  datumName(76) = "Pico De Las Nieves"          '76
  datumName(77) = "Pitcairn Astro 1967"         '77
  datumName(78) = "Prov. So. America 1956"      '78
  datumName(79) = "Prov. So. Chilean 1963"      '79
  datumName(80) = "Puerto Rico"                 '80
  datumName(81) = "Qatar National"              '81
  datumName(82) = "Qornoq"                      '82
  datumName(83) = "Reunion"                     '83
  datumName(84) = "Rome 1940"                   '84
  datumName(85) = "*RT 90 Sweden"               '85
  datumName(86) = "Santo (DOS)"                 '86
  datumName(87) = "Sao Braz"                    '87
  datumName(88) = "Sapper Hill 1943"            '88
  datumName(89) = "Schwarzeck"                  '89
  datumName(90) = "South American 1969"         '90
  datumName(91) = "South Asia"                  '91
  datumName(92) = "Southeast Base - Maderia"    '92
  datumName(93) = "Southwest Base - Azores"     '93
  datumName(94) = "Timbalai 1948"               '94
  datumName(95) = "Tokyo"                       '95
  datumName(96) = "Tristan Astro 1968"          '96
  datumName(97) = "Viti Levu 1916"              '97
  datumName(98) = "Wake-Eniwetok 1960"          '98
  datumName(99) = "WGS 1960"                    '99
  datumName(100) = "WGS 1966"                   '100
  datumName(101) = "WGS 1972"                   '101
  datumName(102) = "WGS 1984"                   '102
  datumName(103) = "Zanderij"                   '103

  datumNumber(1) = 6          '1
  datumNumber(2) = 17         '2
  datumNumber(3) = 16         '3
  datumNumber(4) = 3          '4
  datumNumber(5) = 6          '5
  datumNumber(6) = 6          '6
  datumNumber(7) = 16         '7
  datumNumber(8) = 16         '8
  datumNumber(9) = 16         '9
  datumNumber(10) = 16        '10
  datumNumber(11) = 16        '11
  datumNumber(12) = 3         '12
  datumNumber(13) = 3         '13
  datumNumber(14) = 16        '14
  datumNumber(15) = 5         '15
  datumNumber(16) = 16        '16
  datumNumber(17) = 16        '17
  datumNumber(18) = 16        '18
  datumNumber(19) = 6         '19
  datumNumber(20) = 6         '20
  datumNumber(21) = 5         '21
  datumNumber(22) = 16        '22
  datumNumber(23) = 16        '23
  datumNumber(24) = 16        '24
  datumNumber(25) = 4         '25
  datumNumber(26) = 16        '26
  datumNumber(27) = 16        '27
  datumNumber(28) = 16        '28
  datumNumber(29) = 16        '29
  datumNumber(30) = 16        '30
  datumNumber(31) = 16        '31
  datumNumber(32) = 16        '32
  datumNumber(33) = 5         '33
  datumNumber(34) = 16        '34
  datumNumber(35) = 16        '35
  datumNumber(36) = 16        '36
  datumNumber(37) = 7         '37
  datumNumber(38) = 16        '38
  datumNumber(39) = 2         '39
  datumNumber(40) = 16        '40
  datumNumber(41) = 16        '41
  datumNumber(42) = 7         '42
  datumNumber(43) = 16        '43
  datumNumber(44) = 8         '44
  datumNumber(45) = 5         '45
  datumNumber(46) = 6         '46
  datumNumber(47) = 5         '47
  datumNumber(48) = 5         '48
  datumNumber(49) = 6         '49
  datumNumber(50) = 16        '50
  datumNumber(51) = 4         '51
  datumNumber(52) = 6         '52
  datumNumber(53) = 16        '53
  datumNumber(54) = 6         '54
  datumNumber(55) = 5         '55
  datumNumber(56) = 5         '56
  datumNumber(57) = 5         '57
  datumNumber(58) = 5         '58
  datumNumber(59) = 5         '59
  datumNumber(60) = 5         '60
  datumNumber(61) = 5         '61
  datumNumber(62) = 5         '62
  datumNumber(63) = 5         '63
  datumNumber(64) = 5         '64
  datumNumber(65) = 5         '65
  datumNumber(66) = 13        '66
  datumNumber(67) = 6         '67
  datumNumber(68) = 6         '68
  datumNumber(69) = 6         '69
  datumNumber(70) = 6         '70
  datumNumber(71) = 16        '71
  datumNumber(72) = 14        '72
  datumNumber(73) = 5         '73
  datumNumber(74) = 6         '74
  datumNumber(75) = 1         '75
  datumNumber(76) = 16        '76
  datumNumber(77) = 16        '77
  datumNumber(78) = 16        '78
  datumNumber(79) = 16        '79
  datumNumber(80) = 5         '80
  datumNumber(81) = 16        '81
  datumNumber(82) = 16        '82
  datumNumber(83) = 16        '83
  datumNumber(84) = 16        '84
  datumNumber(85) = 16        '85
  datumNumber(86) = 16        '86
  datumNumber(87) = 16        '87
  datumNumber(88) = 16        '88
  datumNumber(89) = 4         '89
  datumNumber(90) = 18        '90
  datumNumber(91) = 10        '91
  datumNumber(92) = 16        '92
  datumNumber(93) = 16        '93
  datumNumber(94) = 7         '94
  datumNumber(95) = 4         '95
  datumNumber(96) = 16        '96
  datumNumber(97) = 6         '97
  datumNumber(98) = 15        '98
  datumNumber(99) = 19        '99
  datumNumber(100) = 20       '100
  datumNumber(101) = 21       '101
  datumNumber(102) = 22       '102
  datumNumber(103) = 16       '103
  
End Sub
  
