Attribute VB_Name = "Mapstuff"
Option Explicit

'************************************************************************
'
'       Windsway Geodesy Library
'       Version 0.93
'
' The original Visual Basic code was:
'       mapstuff.bas  version 0.91 released 06/25/2002
'       geodesy.cls   version 0.91 released 06/25/2002
'
' The initial developer of the Visual Basic code is < Victor Fraenckel>
' (vfraenc1@nycap.rr.com).
'
' Portions created by < Victor Fraenckel> are
' Copyright (C) 2002 < Windsway Instrument Company>.
'
'
'       Obtained through:
'
'       Windsway Instrument Company
'
' You may get the latest version of this file by email from
' Victor Fraenckel <vfraenc1@nycap.rr.com >
'
' The contents of this file are used with permission, subject to
' the Mozilla Public License Version 1.1 (the "License")' you may
' not use this file except in compliance with the License. You may
' obtain a copy of the License at
' http://www.mozilla.org/MPL/MPL-1.1.html
'
' Software distributed under the License is distributed on an
' "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
' implied. See the License for the specific language governing
' rights and limitations under the License.
''************************************************************************

'************************************************************************
'
' If you modify this file or correct mistakes, I would appreciate
' you dropping me an e-mail to let me know what you think of it and
' let me know about your bug fixes and any additions you have made.
' Reach me at vfraenc1@nycap.rr.com.
'
'Vic Fraenckel 06/08/02
'
'************************************************************************

'************************************************************************
'  revision history
'   date        comment                                       who
'
' 06/03/02      began development                             vhf
' 06/15/02      released version 0.90                         vhf
' 06/17/02      fixed sign problems in DMSToD and DToDMS      vhf
' 08/11/02      fixed same point problem in VInverse          vhf
' 08/11/02      fixed zero distance problem in VDirect        vhf
' 08/20/02      added computation for Gamma (Grid North) to   vhf
'               the functions LL2UTM and UTM2LL
' 08/22/02      changed name of WICISMIN to WICMAXMINS and    vhf
'               WICISSEC to WICMAXSECS
' 08/22/02      added function ParseLatLong                   vhf
' 08/28/02      added function fmtDMS1                        vhf
' 08/28/02      added function LeftDecStr                     vhf
'
'************************************************************************

'************************************************************************
'  module contains the following routines:
'
'   name        last modified date
' ArcCos            06/03/02
' ArcSin            06/03/02
' ArcTan            06/03/02
' ATan2             06/03/02
' ComputeZone       06/03/02
' Converge          07/29/02
' DegToRad          06/03/02
' DMSToD            06/17/02
' DToDMS            06/17/02
' FmtDMS            07/07/02
' FmtDMS1           08/28/02
' GetEllipseStuff   06/03/02
' LeftDecStr        08/28/02
' LL2UTM            06/03/02
' ModAzimuth        06/03/02
' ModLatitude       06/03/02
' ModLongitude      06/03/02
' Modulus           06/03/02
' ParseLatLong      08/22/02
' RadToDeg          06/03/02
' Reciprocal        06/03/02
' Sex2Deg           07/07/02
' Square            06/03/02
' UTM2LL            06/03/02
' VDirect           06/03/02
' VInverse          06/03/02
'
'************************************************************************

'data structures

Public Type DMSStruct
  Degrees As Double
  Minutes As Double
  Seconds As Double
  Sign As Integer
End Type

Public Type EllipseStruct
  major As Double
  minor As Double
  elName As String
End Type

Public Type DatumStruct
  datumNumber As Integer
  datumName As String
End Type

Public Type GeoidStruct
  GeoidNumber As Integer
  GeoidMajor As Double
  GeoidMinor As Double
  GeoidName As String
End Type

Public Type GeoStruct
  Latitude1 As Double
  Longitude1 As Double
  Latitude2 As Double
  Longitude2 As Double
  Azimuth12 As Double
  Azimuth21 As Double
  Distance As Double
  geoid As Integer
End Type

Public Type UTMStruct
  Latitude As Double
  Longitude As Double
  geoid As Integer
  North As Double
  East As Double
  ZoneNumber As Integer
  ZoneLetter As String
  CentralLongitude As Double
  Gamma As Double
  GridSF As Double
End Type

Public Type ZoneStruct
  Latitude As Double
  Longitude As Double
  ZoneNumber As Integer
  ZoneLetter As String
  CentralLongitude As Double
End Type
  
'constants
  Public Const WICCOPYRIGHT = "Copyright (C) 2002"
  Public Const WICCOMPANY = "Windsway Company"
  Public Const WICVERSIONMAJOR As Integer = 0
  Public Const WICVERSIONMINOR As Integer = 91
  
  Public Const WICEPS As Double = 0.00000000000005
  'pi stuff
  Public Const PI As Double = 3.14159265358979
  Public Const WICTWOPI As Double = 2 * PI
  Public Const WICD2R As Double = PI / 180#
  Public Const WICR2D As Double = 180# / PI
    
  'chr() these values produce the sign as a string
  Public Const WICDEGSIGN As Long = 176         'degree sign
  Public Const WICMINSIGN As Long = 39          'minute sign
  Public Const WICSECSIGN As Long = 34          'second sign
  
  Public Const WICNCHOICES As Integer = 22      'num of currently defined ellipsoids
  Public Const WICNDCHOICES As Integer = 103    'num of currently defined datums
  Public Const WICFALSEEAST = 500000#           'for UTM
  Public Const WICFALSENORTH = 0#               'ditto
  Public Const WICFALSESOUTH = 10000000#        'ditto southern hemisphere
  Public Const WICK0 = 0.9996                   'ditto
  Public Const WICCLARK66 As Integer = 5        'clark 1886 ellipsoid
  Public Const WICBESSEL As Integer = 4         'bessel ellipsoid
  Public Const WICINTERNATIONAL As Integer = 16 'international datum
  Public Const WICWGS84 As Integer = 22         'wgs 84 datum
  Public Const WICNAD27 As Integer = 5          'NAD27 datum
  Public Const WICLATNORTH As Integer = 1       'latitude is northern hemisphere
  Public Const WICLATSOUTH As Integer = -1      'latitude is southern hemisphere
  Public Const WICLONGEAST As Integer = 1       'longitude is east
  Public Const WICLONGWEST As Integer = -1      'longitude is west
  Public Const WICNORTH = "N" '                 'north flag
  Public Const WICSOUTH = "S"                   'south flag
  Public Const WICEAST = "E"                    'east flag
  Public Const WICWEST = "W"                    'west flag
  Public Const WICTOKENCT As Integer = 3        'degs, mins secs
  Public Const WICMAXSECS As Double = 59.99999    'really 60 seconds
  Public Const WICMAXMINS As Double = 59.99999    'really 60 minutes
  Public Const WICPOSSIGN As Integer = 1        'positive angle in DMSStruct
  Public Const WICNEGSIGN As Integer = -1       'negative angle in DMSStruct
  
  'BNG Constants
  Public Const WICAIRY As Integer = 2           'Airy ellipsoid
  Public Const WICGRS80 As Integer = 13         'GRS80 ellipsoid
  Public Const WICLAT49 = 49#                   'BNG True origin lat- 49degN
  Public Const WICLON2 = -2#                    'BNG True Origin Long - 2degW
  Public Const WICBNGEAST = 400000#             'For BNG
  Public Const WICBNGNORTH = -100000#           'ditto
  
  'error codes
  Public Const WICNOERROR = 513 + 0              'no error
  Public Const WICLONGERR = 513 + 1              'longitude maximum exceeded
  Public Const WICLATERR = 513 + 2               'latitude maximum exceeded
  Public Const WICBADTOKENCT = 513 + 3           'for parseLatLong
  'error codes for VInverse
  Public Const WICANTIPODAL = 513 + 4            'using antipodal points in VInverse
  Public Const WICBADA12 = 513 + 5               'undefined atan
  Public Const WICBADA21 = 513 + 6               'ditto
  Public Const WICBADGEOID = 513 + 7             'bad geoid selected
  Public Const WICBADLAT1 = 513 + 8              'undefined atan2
  Public Const WICBADLAT2 = 513 + 9              'ditto
  Public Const WICBADLONG1 = 513 + 10            'ditto
  Public Const WICBADLONG2 = 513 + 11            'ditto
  Public Const MSBADLL2BNG = 513 + 12            'error in ll2BNG
  Public Const MSBADBNG2LL = 513 + 13            'error in BNG2LL
  Public Const WICNOTNUMERIC = 513 + 14          'error converting string to num
  Public Const WICINVALIDTOKEN = 513 + 15        'to many tokens returned by parseLatLong
  Public Const WICBADMINSEC = 513 + 16           'minutes or seconds invalid
  Public Const WICBADUTM = 513 + 17              'utm zone finder error
  Public Const WICINVALIDUTMZONE = 513 + 18      'invalid UTM zone
  Public Const WICBADATAN2 = 513 + 19            'general bad atan2
  Public Const WICBADDATUM = 513 + 20            'bad datum index
  'error codes for LL2BNG and BNG2LL
  Public Const WICBADLL2BNG = 513 + 21           'error in ll2BNG
  Public Const WICBADBNG2LL = 513 + 22           'error in BNG2LL
  Public Const WICNOTAIRY = 513 + 23             'geoid not AIRY used for computation
  Public Const WICSAMEPT = 513 + 24              'lat/longs are same point - VInverse
  Public Const WICZERODIST = 513 + 25            'distance is zero - VDirect
  
  'variables
  Public Geo As New Geodesy
  
Public Sub DtoDMS(ByVal angle As Double, _
                   ByRef dms As DMSStruct)

'this subroutine converts angle in decimal degrees to
'degrees, minutes and seconds and returns the result in a DMSDtruct

  Dim a As Double
  Dim d As Double
  Dim m As Double
  Dim s As Double
  Dim sn As Double

  sn = Sgn(angle)
  angle = Abs(angle)
  d = Fix(angle)                      'integer degrees
  m = (angle - d) * 60#               'minutes
  s = (m - Fix(m)) * 60#              'seconds
  
  If s > WICMAXSECS Then                'really 60 seconds
    s = 0#                            'bump up
    m = m + 1#                        'ditto minutes
  End If
  If m > WICMAXMINS Then                'really 60 minutes
    m = 0#                            'bump up
    d = d + 1#                        'ditto minutes
  End If
  
  dms.Degrees = d                     'store degrees
  dms.Minutes = Fix(m)                'store
  dms.Seconds = s                     'ditto
  dms.Sign = sn
  
End Sub

Public Function DMStoD(dms As DMSStruct) As Double

'this function converts an angle in dms format to the
'corresponding decimal degrees.
'NOTE: sign = -1.0 for negative
'           = 0.0 for zero
'           = 1.0 for positive

  Dim sn As Double
  Dim d As Double
  Dim m As Double
  Dim s As Double
  
  sn = dms.Sign
  
  d = Abs(dms.Degrees)
  m = Abs(dms.Minutes)
  s = Abs(dms.Seconds)
  
  If s > WICMAXSECS Then
    s = 0#
    m = m + 1#
  End If
  
  If m > WICMAXMINS Then
    m = 0#
    d = d + 1#
  End If
  
  DMStoD = sn * (d + m / 60# + s / 3600#)
                
End Function

Public Function ArcTan(ByVal a As Double) As Double
'function computes the ArcTangent of the angle a.
'the angle a is assumed to be expressed in RADIANS

  ArcTan = Atn(a)
  
End Function

Public Function ArcSin(ByVal a As Double) As Double
'function computes the ArcSine of the angle a
'the angle a is assumed to be in RADIANS

  If Abs(a) <> 1 Then
    ArcSin = Atn(a / Sqr(-a * a + 1))
  Else
    ArcSin = IIf(a = 1, Atn(1) * 2, Atn(1) * 6)
  End If
  
End Function

Public Function ArcCos(ByVal a As Double) As Double
'function computes the ArcCosine of the angle a
'the angle a is assumed to be expressed in RADIANS
  
  If Abs(a) <> 1 Then
      ArcCos = Atn(-a / Sqr(-a * a + 1)) + 2 * Atn(1)
   Else
      ArcCos = IIf(a = 1, 0, Atn(1) * 4)
   End If

End Function

Public Function Atan2(ByVal y As Double, ByVal x As Double) As Double
'function computes the ArcTangent of the angle formed by
'the vectors x and y.
  
  If (x < 0) Then
    Atan2 = ArcTan(y / x) + PI
  End If
  
  If x > 0 And y >= 0 Then
    Atan2 = ArcTan(y / x)
  End If
  
  If x > 0 And y < 0 Then
    Atan2 = ArcTan(y / x) + 2# * PI
  End If
  
  If x = 0 And y > 0 Then
    Atan2 = PI / 2#
  End If
  
  If x = 0 And y < 0 Then
    Atan2 = 3# * PI / 2#
  End If
  
  If x = 0 And y = 0 Then
    Atan2 = 0#
  End If
  
End Function


Public Function Modulus(ByVal x As Double, _
                        ByVal y As Double) As Double
'return the floating point modulus
  
  Modulus = x - y * Int(x / y)
  
End Function

Public Function ModLatitude(ByVal lat As Double) As Double

'returns the value of the argument (in radians) constrained to
'range of -90.0 <= x <= +90.0. Value returned in radians

  ModLatitude = Modulus(lat + PI / 2#, PI) - PI / 2#
  'ModLatitude = (lat + PI / 2#) Mod PI - PI / 2#
End Function

Public Function ModLongitude(ByVal lon As Double) As Double
'returns the value of the argument (in radians) constrained to
'range of -180.0 <= x <= +180.0. Value returned in radians

  ModLongitude = Modulus(lon + PI, 2# * PI) - PI
  'ModLongitude = (lon + PI) Mod (2# * PI) - PI
  

End Function

Public Function ModAzimuth(ByVal az As Double) As Double
'returns the value of the argument (in radians) constrained to
'range of 0.0 <= x <= +360.0. Value returned in radians
    
  ModAzimuth = Modulus(az, 2# * PI)
  'ModAzimuth = az Mod 2# * PI
  
End Function

Public Function Reciprocal(ByVal angle As Double) As Double
'this function finds the reciprocal of angle.
'angle is reduced to range 0-360 first and then
'the reciprocal is computed
'angle is assumed to be in decimal degrees
'the result is decimal degrees

                    'algorithm is from Alan

  
  While angle < 0#        'handle negative values
    angle = angle + 360#
  Wend

  While angle > 360#      'handle big stuff
    angle = angle - 360#
  Wend
 
  If angle <= 180# Then
    angle = angle + 200# - 20#
  Else
    angle = angle + 20# - 200#
  End If
  
  Reciprocal = angle

End Function

Public Function DegToRad(ByVal angle As Double) As Double
'converts angle as degrees to angle as radians

  DegToRad = angle * WICD2R
  
End Function

Public Function RadToDeg(ByVal angle As Double) As Double
'converts angle as radians to angle as degrees

  RadToDeg = angle * WICR2D
  
End Function

Public Function Square(ByVal x As Double) As Double
'function computes the square of x

  Square = x * x
  
End Function

Public Function ComputeZone(ByRef zone As ZoneStruct) As Integer

'This function computes UTM zone information for a given lat/long pair
'The method returns WICNOERROR for a valid computation else it returns
'WICBADUTM which indicates the latitude is outside the range for UTMs as
'UTM zones are confined to the latitude range -80 thru +84 degrees.
'
  Dim lon As Double
  Dim lat As Double
  Dim zn As Integer
  Dim znd As Double
  Dim zlet As String

  lon = ModLongitude(DegToRad(zone.Longitude))
  lat = ModLatitude(DegToRad(zone.Latitude))
  lon = RadToDeg(lon)
  lat = RadToDeg(lat)

  If (lat > 84#) Or (lat < -80#) Then
    ComputeZone = WICBADUTM
    Exit Function
  End If

  zn = Fix((lon + 180#) / 6# + 1#)        'zone

  If (lat >= 56#) And (lat < 64) And (lon >= 3#) And (lon < 12#) Then
    zn = 32
  End If
  
  ' zones for Svalbard
  zlet = "Z"
  If (lat >= 72#) And (lat < 84#) Then
      If (lon >= 0#) And (lon < 9#) Then
        zn = 31
      ElseIf lon >= 9# And lon < 21# Then
        zn = 33
      ElseIf lon >= 21# And lon < 33# Then
        zn = 35
      ElseIf lon >= 33# And lon < 42# Then
        zn = 37
      End If
  End If
  
  ' determine zone letter
  If lat <= 84# And lat >= 72# Then
    zlet = "X"
  ElseIf lat < 72# And lat >= 64# Then
    zlet = "W"
  ElseIf lat < 64# And lat >= 56# Then
    zlet = "V"
  ElseIf lat < 56# And lat >= 48# Then
    zlet = "U"
  ElseIf lat < 48# And lat >= 40# Then
    zlet = "T"
  ElseIf lat < 40# And lat >= 32# Then
    zlet = "S"
  ElseIf lat < 32# And lat >= 24# Then
    zlet = "R"
  ElseIf lat < 24# And lat >= 16# Then
    zlet = "Q"
  ElseIf lat < 16# And lat >= 8# Then
    zlet = "P"
  ElseIf lat < 8# And lat >= 0# Then
    zlet = "N"
  ElseIf lat < 0# And lat >= -8# Then
    zlet = "M"
  ElseIf lat < -8# And lat >= -16# Then
    zlet = "L"
  ElseIf lat < -16# And lat >= -24# Then
    zlet = "K"
  ElseIf lat < -24# And lat >= -32# Then
    zlet = "J"
  ElseIf lat < -32# And lat >= -40# Then
    zlet = "H"
  ElseIf lat < -40# And lat >= -48# Then
    zlet = "G"
  ElseIf lat < -48# And lat >= -56# Then
    zlet = "F"
  ElseIf lat < -56# And lat >= -64# Then
    zlet = "E"
  ElseIf lat < -64# And lat >= -72# Then
    zlet = "D"
  ElseIf lat < -72# And lat >= -80# Then
    zlet = "C"
  End If
  
  zone.ZoneNumber = zn
  zone.CentralLongitude = (zn - 1) * 6 - 180# + 3
  zone.ZoneLetter = zlet

  ComputeZone = WICNOERROR

End Function

Public Function GetEllipseStuff(ByRef es As EllipseStruct, _
                                ByVal geoid As Integer) As Integer
                                
'this function extracts the requested ellipsoid information from
'the Private class array and makes it available to the user's
'application. It returns WICNOERROR on success or WICBADGEOID
'if the requested ellipse does not exist. Valid geoid numbers
'range from 1 to 22. The maximum geoid number may be extracted
'from the class using the Get function Choices().
                                
  Dim c As Integer
  
  c = Geo.Choices
  
  If ((geoid < 1) Or (geoid > c)) Then
    GetEllipseStuff = WICBADGEOID                   'flag error
    Exit Function
  End If
  
  es.major = Geo.eMajor(geoid)
  es.minor = Geo.eMinor(geoid)
  es.elName = Geo.ename(geoid)
  
  GetEllipseStuff = WICNOERROR
  
End Function
Public Function GetDatumStuff(ByRef ds As DatumStruct, _
                              ByVal index) As Integer
                              
'this function extracts the datum name and it's cooresponding
'index from the private class arrays datumName and datumNumber
'and make it available to the user's application. The function
'returns WICNOERROR if successful else it returns WICBADDATUM
'if it is passed an index out of range of 1 to Geo.DChoices

  Dim c As Integer
  
  c = Geo.DChoices
  If index < 1 Or index > c Then
    GetDatumStuff = WICBADDATUM
    Exit Function
  End If
  
  ds.datumName = Geo.dName(index)
  ds.datumNumber = Geo.dNumber(index)
  GetDatumStuff = WICNOERROR
  
End Function
Public Function Converge(ByRef gs As GeoStruct) As Integer
'this function computes the convergence of meridians between
'two parallels of latitude. The convergence is returned in
'meters in gs.distance.

'Negative convergence implies meridians converge toward the pole
'or abs(latitude1 is nearer the pole than abs(latitude2)

'Positive convergence implies meridians diverge towarf the equator
'or abs(latitude1 is nearer the equator than abs(Latitude2)

'the longitude order makes no difference

'The function returns WICNOERROR if completed successfully
'or WICBADGEOID if unable to find data for gs.geoid

  Dim eflag As Integer
  Dim a0, b0 As Double
  Dim flat As Double
  Dim esq As Double
  Dim deltaL, deltalambda As Double
  Dim lambda1, lambda2 As Double
  Dim phi1, phi2 As Double
  Dim sinphi1, cosphi1 As Double
  Dim sinphi2, cosphi2 As Double
  Dim l1, l2 As Double
  Dim term1, term2 As Double
  
  Dim es As EllipseStruct
  
  eflag = GetEllipseStuff(es, gs.geoid)
  If eflag = WICBADGEOID Then
    Converge = WICBADGEOID
    Exit Function
  End If

  a0 = es.major
  b0 = es.minor
  flat = (a0 - b0) / a0
  esq = 2# * flat - Square(flat) 'eccentricity squared
  
  lambda1 = DegToRad(gs.Longitude1)
  lambda2 = DegToRad(gs.Longitude2)
  deltalambda = Abs(lambda2 - lambda1)  'order does not matter
  phi1 = DegToRad(gs.Latitude1)
  phi2 = DegToRad(gs.Latitude2)
  sinphi1 = Sin(phi1)
  cosphi1 = Cos(phi1)
  sinphi2 = Sin(phi2)
  cosphi2 = Cos(phi2)
  
  'first compute length of arc at latitude1
  term1 = a0 * cosphi1 * deltalambda
  term2 = Sqr(1# - esq * Square(sinphi1))
  l1 = term1 / term2  'arc length in meters
  
  'now compute length of arc at latitude2
  term1 = a0 * cosphi2 * deltalambda
  term2 = Sqr(1# - esq * Square(sinphi2))
  l2 = term1 / term2  'arc length in meters
  deltaL = l2 - l1
  
  gs.Distance = deltaL
  Converge = WICNOERROR
  
End Function

Public Function LL2UTM(ByRef utm As UTMStruct) As Integer
'This method converts a latatude/longitude pair to the corresponding
'UTM grid coordinates.
'User supplies:
'   Latitude
'   Longitude
'   Geoid
'in a UTMStructure
'Method returns:
'    UTM Easting
'    UTM Northing
'    UTM Zone Number
'    UTM Zone Letter
'    UTM Central Longitude
'    UTM Grid Scale Factor
'in the same UTMStructure

'The method impliments the algorithm as published in Map Projections -
'a Working Manual, USGS Professional paper 1395.

'the function returns WICNOERROR if successful, WICBADGEOID if unable
'to get the requested geoid data or WICINVALIDUTMZONE for invalid zone
'information.

  Dim eflag, geoid As Integer
  Dim a0, b0 As Double
  Dim m, m0 As Double
  Dim e2, e4, e6 As Double
  Dim eprime2, eprime4, eprime6, eprime8 As Double
  Dim flat As Double
  Dim a As Double
  Dim n, n1, t1  As Double
  Dim theta, rtheta As Double
  Dim a1, a12, a13, a14, a15, a16 As Double
  Dim tanphi, tan2phi, ets, c1, c2, c3, c4, c5, l, ls, rcm, conv As Double
  
  Dim lambda As Double
  Dim rlambda As Double
  Dim dl, dl3, dl5, dl7 As Double
  
  Dim lambda0 As Double
  Dim rlambda0 As Double
  Dim phi As Double
  Dim rphi As Double
  Dim sinphi, cosphi As Double
  Dim cos2phi, cos4phi, cos6phi, cos8phi As Double
  Dim term1 As Double
  Dim term2 As Double
  Dim term3 As Double
  Dim term4 As Double
  Dim x As Double
  Dim y As Double
  Dim k As Double
  Dim zone As ZoneStruct
  Dim es As EllipseStruct

  geoid = utm.geoid
  eflag = GetEllipseStuff(es, geoid)
  If (eflag = WICBADGEOID) Then
      LL2UTM = WICBADGEOID
      Exit Function
  End If

  a0 = es.major
  b0 = es.minor

  zone.Longitude = utm.Longitude
  zone.Latitude = utm.Latitude

  eflag = ComputeZone(zone)
  If eflag <> WICNOERROR Then
    LL2UTM = WICINVALIDUTMZONE
    Exit Function
  End If
  
  lambda0 = zone.CentralLongitude

  'return central longitude information
  utm.CentralLongitude = zone.CentralLongitude
  rcm = DegToRad(utm.CentralLongitude)
  utm.ZoneNumber = zone.ZoneNumber
  utm.ZoneLetter = zone.ZoneLetter

  lambda = utm.Longitude
  rlambda0 = DegToRad(lambda0)
  rlambda = DegToRad(lambda)
  phi = utm.Latitude
  rphi = DegToRad(phi)
  sinphi = Sin(rphi)
  cosphi = Cos(rphi)
  cos2phi = Square(cosphi)
  cos4phi = Square(cos2phi)
  cos6phi = cos2phi * cos4phi
  cos8phi = Square(cos4phi)
  tanphi = sinphi / cosphi
  tan2phi = Square(tanphi)
  dl = rlambda - rcm
  l = dl * cosphi
  ls = Square(l)
  dl3 = dl * Square(dl)
  dl5 = dl3 * Square(dl)
  dl7 = dl5 * Square(dl)
  
  'SIGN CONVENTION:
  '     north latitudes and east longitudes are positive

  'compute flatting from geoid ellipse constants
  flat = (a0 - b0) / a0

  'compute eccentricity squared
  e2 = 2# * flat - Square(flat)          'e^2 first eccentricity squared
  e4 = Square(e2)                        'e^4
  e6 = e4 * e2                           'e^6

  theta = utm.Latitude
  rtheta = DegToRad(theta)

  'compute e prime from equation (8-12)
  eprime2 = e2 / (1# - e2)               'second eccentricity squared
  eprime4 = Square(eprime2)
  eprime6 = eprime2 * eprime4
  eprime8 = Square(eprime4)
  
  'compute the convergence - grid north
  'ets = eprime2 * Square(cosphi)
  'c2 = -tn
  'c3 = (1# + 3# * ets + 2# * Square(ets)) / 3#
  'c5 = (2# - ts) / 15#
  'conv = c2 * l * (1# + ls * (c3 + c5 * ls)) 'grid north
  
  c2 = sinphi
  c3 = sinphi * cos2phi / 3# * (1# + 3# * eprime2 + 2# * eprime4 * cos4phi)
  c4 = 2# - tan2phi + 15# * eprime2 * cos2phi + 35# * eprime4 * cos4phi - _
          15# * tan2phi * eprime2 * cos2phi + 33# * eprime6 * cos6phi - _
          50# * tan2phi * eprime4 * cos4phi + 11# * eprime8 * cos8phi - _
          60# * tan2phi * eprime6 * cos6phi
  c4 = sinphi * cos4phi / 15# * c4
  c5 = sinphi * cos6phi / 315# * (7# - 26# * tan2phi + 2# * Square(tan2phi))
  conv = dl * c2 + dl3 * c3 + dl5 * c4 + dl7 * c5
  
  a = a0

  'compute N using equation (4-20 pg 61)
  n1 = a / Sqr(1# - e2 * Square(Sin(rtheta)))

  'compute T using equation (8-13)
  t1 = Square(Tan(rtheta))

  'compute C using equation (8-14)
  c1 = eprime2 * Square(Cos(rtheta))

  'compute A using equation (8-15)
  a1 = (rlambda - rlambda0) * Cos(rtheta)
  a12 = Square(a1)                          'a^2
  a13 = a12 * a1                         'a^3
  a14 = a13 * a1                         'a^4
  a15 = a14 * a1                         'a^5
  a16 = a15 * a1                         'a^6

  m0 = 0#

  'compute M using equation (3-21)
  'this expression is more general for any geoidlipsoid

  term1 = rphi * (1# - e2 / 4# - 3# * e4 / 64# - 5# * e6 / 256#)
  term2 = Sin(2# * rphi) * (3# * e2 / 8# + 3# * e4 / 32# + _
              45# * e6 / 1024#)
  term3 = Sin(4# * rphi) * (15# * e4 / 256# + 45# * e6 / 1024#)
  term4 = Sin(6# * rphi) * 35# * e6 / 3072#
  m = a * (term1 - term2 + term3 - term4)


  'compute x (east) from equation (8-9)
  term1 = WICK0 * n1
  term2 = a1 + (1# - t1 + c1) * a13 / 6#
  term3 = (5# - 18# * t1 + Square(t1) + 72# * c1 - 58# * eprime2) * _
           a15 / 120#
  x = term1 * (term2 + term3)

  'compute y (north) from equation (8-10)
  term1 = m - m0
  term2 = n1 * Tan(rphi)
  term3 = a12 / 2# + (5# - t1 + 9# * c1 + 4# * Square(c1)) * a14 / 24#
  term4 = (61# - 58# * t1 + Square(t1) + 600# * c1 - 330# * eprime2) * _
             a16 / 720#
  y = WICK0 * (term1 + term2 * (term3 + term4))

  'compute k from equation (8-11)
  k = WICK0 * (1# + (1# + eprime2 * Cos(rphi) * Cos(rphi)) * _
             Square(x) / (2# * Square(WICK0) * Square(n1)))

  'return easting and northing and local scale factor
  x = x + WICFALSEEAST
  utm.East = x

  If (Sgn(utm.Latitude) = WICLATSOUTH) Then
      y = y + WICFALSESOUTH
  Else
      y = y + WICFALSENORTH
  End If
  
  utm.North = y
  utm.GridSF = k
  utm.ZoneLetter = zone.ZoneLetter
  utm.Gamma = RadToDeg(conv)      'return as decimal degrees
  LL2UTM = WICNOERROR                      'flag no error


End Function

Public Function UTM2LL(ByRef utm As UTMStruct) As Integer
'This method converts a UTM coordinate pair to the corresponding
'latitude/longitude.
'
'this methoid impliments the algorithm as published in
' Map Projections - a working manual, USGS Professional paper
' 1395.
'User supplies:
'   Geoid
'   UTM Easting
'   UTM Northing
'   UTM Zone Letter
'   UTM Zone Number
'in a UTMStructure

'Method returns:
'   UTM Latitude
'   UTM Longitude;
'   UTM Central Longitude
'in the same UTMStructure.

'the method returns WICNOERROR if successful else it returns
'WICBADGEOID

 'this subroutine impliments the algorithm as published in
 'Map Projections - a working manual, USGS Professional paper
 '1395.
    
    Dim eflag As Integer
    Dim a0, b0 As Double
    Dim m, m0, n, n2, n3, n4, G, sigma As Double
    Dim e2, eprime2 As Double
    Dim flat As Double
    Dim e1, e12, e13, e14 As Double
    Dim c1, n1, r1 As Double
    Dim d, d2, d3, d4, d5, d6 As Double
    Dim a, u As Double
    Dim tmp As Double
    Dim phi, phi1, fpphi As Double
    Dim term1, term2, term3, term4, term5, term6 As Double
    Dim lambda, rlambda As Double
    Dim utmlat As Double
    Dim utmlon As Double
    Dim East, North As Double
    Dim c2, c3, c5 As Double
    Dim s1, s2, s4, s6, s8 As Double
    Dim x1, x2, x3, x4, x5, x7 As Double
    Dim p1, psi, psi1, psi2, psi3, psi4 As Double
    Dim t, t1, t2, t3, t4, t5, t6 As Double
    Dim conv, sinphi, cosphi As Double
    Dim geoid As Integer
    Dim es As EllipseStruct
    
    'get ellipse number for constants
    geoid = utm.geoid
    eflag = GetEllipseStuff(es, geoid)
    
    If (eflag = WICBADGEOID) Then
      UTM2LL = WICBADGEOID
      Exit Function
    End If

    a0 = es.major
    b0 = es.minor
    
    
    East = utm.East - WICFALSEEAST
    If (utm.ZoneLetter < "N") Then            'southern hemisphere
      North = utm.North - WICFALSESOUTH
     Else
      North = utm.North - WICFALSENORTH
    End If
    'SIGN CONVENTION:
    '     north latitudes and east longitudes are positive

    'compute flatting from ellipse constants
    flat = (a0 - b0) / a0

    'compute first excentricity squared
    e2 = 2# * flat - Square(flat)
    'second eccentricity squared equation 8-12
    eprime2 = e2 / (1# - e2)

    'now compute m0 using equation (3.22)
    m0 = 0#                                  'for UTM

    'compute m from equation (8-20)
    m = m0 + North / WICK0
  
    'compute e1 from equation(3-24)
    e1 = (1# - Sqr(1# - e2)) / (1# + Sqr(1# - e2))
    a = a0                                 'major axis in meters

    'compute u using equation (7-19)
    u = m / (a * (1 - e2 / 4 - 3 * e2 * e2 / 64 - 5 * e2 * e2 * e2 / 256))
    'NOTE:u is in radians

    e12 = e1 * e1
    e13 = e12 * e1
    e14 = e13 * e1

    'compute phi1 using equation (3-26)
    term1 = u
    term2 = (3 * e1 / 2 - 27 * e13 / 32) * Sin(2 * u)
    term3 = (21 * e12 / 16 - 55 * e14 / 32) * Sin(4 * u)
    term4 = (151 * e13 / 96) * Sin(6 * u)
    term5 = (1097 * e14 / 512) * Sin(8 * u)
    'compute the footprint latitude
    phi1 = term1 + term2 + term3 + term4 + term5
    'NOTE:phi1 is in radians

    '{compute c1 using equation (8-21)
    c1 = eprime2 * Cos(phi1) * Cos(phi1)

    'compute t using equation (8-22)
    t = Tan(phi1) * Tan(phi1)

    '{compute n1 from equation (8-23)}
    'radius of curvature of the ellipsoid in the prime vertical
    n1 = a / Sqr((1 - e2 * Sin(phi1) * Sin(phi1)))
    'compute the radius of curvature of the ellipsoid in the plane of the meridian
    n = 1# - e2 * Square(Sin(phi1))
    n2 = Square(n)
    n3 = n * n2
    p1 = a * (1# - e2) / Sqr(n3)
    
    psi1 = n1 / p1
    psi2 = Square(psi1)
    psi3 = psi2 * psi1
    psi4 = Square(psi2)
    
    'compute the convergence
    'method from GDA Tech Manual chapter 5
    t1 = Tan(phi1)
    t2 = Square(t1)
    t3 = t1 * t2
    t4 = Square(t2)
    t5 = t4 * t1
    t6 = t2 * t4
    
    x1 = East / (WICK0 * n1)
    x3 = x1 * Square(x1)
    x5 = x3 * x2
    x7 = x5 * x2
    
    term1 = -t1 * x1
    term2 = (t1 * x3 / 3#) * (-2# * psi2 + 3# * psi + Square(t1))
    term3 = (t1 * x5 / 15#) * (psi4 * (11# - 24# * t2) - 3# * psi3 * (8# - 23# * t2) + 5# * psi2 * _
            (3# - 14# * t2) + 30 * psi1 * t2 + 3# * t4)
    term4 = (t1 * x7 / 315#) * (17# + 77# * t2 + 105# * t4 + 45# * t6)
    conv = term1 + term2 + term3 + term4  'radians
    utm.Gamma = RadToDeg(conv)            'return decimal degrees
    
      'compute r1 from equation (8-24)
    tmp = (1# - e2 * Sin(phi1) * Sin(phi1))
    tmp = Sqr(tmp * tmp * tmp)
    r1 = a * (1# - e2) / tmp

    'compute d from equation (8-25)
    d = East / (n1 * WICK0)
    d2 = d * d
    d3 = d2 * d
    d4 = d3 * d
    d5 = d4 * d
    d6 = d5 * d

    'compute phi from equation (8-17)
    term1 = phi1
    term2 = (n1 * Tan(phi1) / r1)
    term3 = d2 / 2# - (5# + 3# * t + 10# * c1 - 4# * Square(c1) - _
            9# * eprime2) * d4 / 24#
    term4 = (61# + 90# * t + 298# * c1 + 45# * Square(t) - _
            252# * eprime2 - 3# * Square(c1)) * d6 / 720#
    phi = term1 - term2 * (term3 + term4)  'in radians

    utm.Latitude = ModLatitude(phi)        'constrain to -90 to 90
    utm.Latitude = RadToDeg(utm.Latitude)

    'compute lambda from equation (8-18)
    utm.CentralLongitude = (utm.ZoneNumber - 1) * 6 - 180# + 3
    term1 = utm.CentralLongitude
    term2 = d - (1# + 2# * t + c1) * d3 / 6#
    term3 = (5# - 2# * c1 + 28# * t - 3# * Square(c1) + 8# * eprime2 + _
            24# * Square(t)) * d5 / 120#
    lambda = term1 + RadToDeg((term2 + term3) / Cos(phi1))
    
    utm.Longitude = lambda

    UTM2LL = WICNOERROR

End Function


Public Function VInverse(ByRef gs As GeoStruct) As Integer

'Solution of the geodetic inverse problem after T. Vincenty}
'function is effective of any azimuth and any distance short
'of antipodal. Start and end lat/long must not be a geograpic
'pole. Latitudes and longitudes are positive north and east.
'Given: latitude and longitude of two points, VInverse will
'compute the ellipsoidal distance (in meters) and forward and
'reverse azimuths

'user supplies:
' Point 1 Latitude
' Point 1 Longitude
' Point 2 Latitude
' Point 2 Longitude
' Required geoid number
'in a GeoStructure
'function returns:
' Azimuth of point 2 from point 1 - Azimuth12
' Azimuth of point 1 from point 2 - Azimuth21
' Distance between points in Meters
'in the same structure

'function returns WICNOERROR if successful else it returns error code
'ERRORS RETURNED
'       WICANTIPODAL - for antipodal lat/longs
'       WICBADGEOID - for bad geoid number
'       WICBADA12 - for a bad az12 calculation of ATan2(az12)
'       WICBADA21 - for a bad az21 calculation of ATan2(az21
'       WICSAMEPT - lat1/long1 and lat2/long2 same point
'         NOTE: this error flag is not fatal - function returns
'               0 for azimuths and distance.

  Dim eflag As Integer
  Dim lat1 As Double
  Dim long1 As Double
  Dim lat2 As Double
  Dim long2 As Double
  Dim a, a0 As Double
  Dim b, b0 As Double
  Dim flat As Double
  Dim c, r, s As Double
  Dim u2, x As Double
  Dim sinu1, sinu2 As Double
  Dim cosu1, cosu2 As Double
  Dim tanu1, tanu2 As Double
  Dim lambda, lambda0, testlambda, omega As Double
  Dim sinsigma, sin2sigma, s2s, ss As Double
  Dim cossigma, cos2sigmam, c2sm, cs As Double
  Dim tansigma As Double
  Dim sinalpha As Double
  Dim cosalpha As Double
  Dim dsigma As Double
  Dim alpha12, alpha21 As Double
  Dim tanalpha12, tanalpha21 As Double
  Dim es As EllipseStruct

  Dim tl1, tl2, tld As Double
  
  lat1 = DegToRad(gs.Latitude1)
  lat2 = DegToRad(gs.Latitude2)
  long1 = DegToRad(gs.Longitude1)
  long2 = DegToRad(gs.Longitude2)
  
  If lat1 = lat2 And Abs(long1 - long2) = PI Then
    VInverse = WICANTIPODAL
    Exit Function
  End If
  
  tl1 = gs.Latitude1
  tl2 = gs.Latitude2
  tld = tl1 - tl2
  'same point detection
  If Abs(gs.Latitude1 - gs.Latitude2) < 0.000001 And _
     Abs(gs.Longitude1 - gs.Longitude2) < 0.000001 Then
    gs.Azimuth12 = 0#
    gs.Azimuth21 = 0#
    gs.Distance = 0#
    VInverse = WICSAMEPT
    Exit Function
  End If
  
  eflag = GetEllipseStuff(es, gs.geoid)
  If eflag = WICBADGEOID Then
    VInverse = WICBADGEOID
    Exit Function
  End If

  a0 = es.major
  b0 = es.minor
  flat = (a0 - b0) / a0
  r = 1# - flat

  tanu1 = r * Tan(lat1)
  tanu2 = r * Tan(lat2)

  x = Atn(tanu1)
  cosu1 = Cos(x)
  sinu1 = Sin(x)

  x = Atn(tanu2)
  cosu2 = Cos(x)
  sinu2 = Sin(x)

  omega = long2 - long1
  lambda = omega

  Do
    testlambda = lambda
    s2s = Square(cosu2 * Sin(lambda)) + _
            Square(cosu1 * sinu2 - sinu1 * cosu2 * Cos(lambda))
    ss = Sqr(s2s)
    cs = sinu1 * sinu2 + cosu1 * cosu2 * Cos(lambda)
    tansigma = ss / cs
    sinalpha = cosu1 * cosu2 * Sin(lambda) / ss
      x = ArcSin(sinalpha)
    cosalpha = Cos(x)
    c2sm = cs - (2# * sinu1 * sinu2 / Square(cosalpha))
    c = flat / 16# * Square(cosalpha) * _
          (4# + flat * (4# - 3# * Square(cosalpha)))
    lambda = omega + (1# - c) * _
              flat * sinalpha * (ArcSin(ss) + c * ss * (c2sm + _
                c * cs * (-1# + 2# * Square(c2sm))))
  Loop Until (Abs(testlambda - lambda) <= WICEPS)
  
  u2 = Square(cosalpha) * (Square(a0) - Square(b0)) / Square(b0)
  a = 1# + (u2 / 16384#) * (4096# + u2 * _
          (-768# + u2 * (320# - 175# * u2)))
  b = (u2 / 1024#) * (256# + u2 * (-128# + u2 * (74# - 47# * u2)))

  dsigma = b * ss * (c2sm + (b / 4#) * (cs * (-1# + 2# * Square(c2sm)) - _
          (b / 6#) * c2sm * (-3# + 4# * Square(ss)) * (-3# + 4# * Square(c2sm))))

  s = b0 * a * (ArcSin(ss) - dsigma)

  alpha12 = Atan2(cosu2 * Sin(lambda), _
    (cosu1 * sinu2 - sinu1 * cosu2 * Cos(lambda)))
      
  alpha21 = Atan2(cosu1 * Sin(lambda), _
      (-sinu1 * cosu2 + cosu1 * sinu2 * Cos(lambda))) - PI

  alpha12 = ModAzimuth(alpha12)       'constrain value
  alpha21 = ModAzimuth(alpha21)       'ditto
  gs.Azimuth12 = RadToDeg(alpha12)    'convert to decimal degrees
  gs.Azimuth21 = RadToDeg(alpha21)    'ditto
  gs.Distance = s                                                                      'distance in metere
  VInverse = WICNOERROR                 'return success

End Function

Public Function VDirect(ByRef gs As GeoStruct) As Integer
'Solution of the geodetic direct problem after T. Vincenty
'Given: latitude and longitude of a point and the geodetic
'azimuth and ellipsoidal distance to a second point, VDirect
'will compute the latitude and longitude of the second point
'and the reverse azimuth

'user supplies:
' Point 1 Latitude
' Point 1 Longitude
' Azimuth of point 2 from point 1 - Azimuth12
' distance of point 2 from point 2 in meters
' Required geoid number
'in a GeoStructure
'function returns:
' Latitude of point 2 - Latitude2
' Longitude of point 2 - Longitude2
' Azimuth of point 1 from point 2 - Azimuth21
'in the same structure

'function returns:
'   WICNOERROR for success
'   WICBADGEOID if retrival of ellipsoid data fails
'   WICZERODIST if distance to second point is zero
'     NOTE: this error is not fatal. VDirect returns
'           0 for azimuth21 and lat1/long1 for lat2/long2

  Dim eflag As Integer
  Dim lat1, lat2, long1, long2 As Double
  Dim az12, az21 As Double
  Dim s, a0, b0 As Double
  Dim flat, r As Double
  Dim u1, tanu1, sinu1, cosu1, tansigma1, sinalpha, cosalpha As Double
  Dim a, b, c, usqr As Double
  Dim sigma, sigma1, deltasigma, twosigmam, lastsigma As Double
  Dim costwosigmam As Double
  Dim term1, term2, term3, term4 As Double
  Dim l, lambda, tanlambda, omega As Double
  Dim ss, cs, c2sm As Double
  Dim es As EllipseStruct

  lat1 = DegToRad(gs.Latitude1)
  long1 = DegToRad(gs.Longitude1)
  az12 = DegToRad(gs.Azimuth12)
  s = gs.Distance
  
  'handle zero distance
  If s = 0# Then
    gs.Latitude2 = gs.Latitude1
    gs.Longitude2 = gs.Longitude1
    gs.Azimuth21 = 0#
    VDirect = WICZERODIST
    Exit Function
  End If
  
  eflag = GetEllipseStuff(es, gs.geoid)

  If (eflag = WICBADGEOID) Then
    VDirect = WICBADGEOID
    Exit Function
  End If
  
  a0 = es.major
  b0 = es.minor
  flat = (a0 - b0) / a0
  r = 1# - flat
  tanu1 = r * Tan(lat1)
  
  tansigma1 = tanu1 / Cos(az12)           'eq 1
  
  u1 = ArcTan(tanu1)
  sinu1 = Sin(u1)
  cosu1 = Cos(u1)
  
  sinalpha = cosu1 * Sin(az12)            'eq 2
  cosalpha = Sqr(1# - Square(sinalpha))
  
  usqr = Square(cosalpha) * (Square(a0) - Square(b0)) / Square(b0)
  
  term1 = usqr / 16384#
  term2 = 4096# + usqr * (-768# + usqr * (320# - 175# * usqr))
  a = 1# + term1 * term2
  b = usqr / 1024# * (256# + usqr * (-128# + usqr * (74# - 47# * usqr))) 'eq 4
                                          
  sigma = s / (b0 * a)
  sigma1 = ArcTan(tansigma1)
  
  Do
    lastsigma = sigma
    twosigmam = 2# * sigma1 + sigma      'eq 5
    ss = Sin(sigma)                 'sin(sigma)
    cs = Cos(sigma)                 'cos(sigma)
    c2sm = Cos(twosigmam)           'cos(twosigmam)

    deltasigma = b * ss * (c2sm + b / 4# * (cs * (-1# + 2# * Square(c2sm)) - _
      b / 6# * c2sm * (-3# + 4# * Square(ss)) * _
        (-3# + 4# * Square(c2sm))))                   'eq 6

    sigma = s / (b0 * a) + deltasigma                  'eq 7

  Loop Until (Abs(sigma - lastsigma) <= WICEPS)

  twosigmam = 2# * sigma1 + sigma                    'eq 5
  ss = Sin(sigma)                 'sin(sigma)
  cs = Cos(sigma)                 'cos(sigma)
  c2sm = Cos(twosigmam)           'cos(twosigmam)
  term1 = sinu1 * cs + cosu1 * ss * Cos(az12)
  term2 = Square(sinalpha) + Square(sinu1 * ss - cosu1 * cs * Cos(az12))
  term3 = r * Sqr(term2)
  lat2 = Atan2(term1, term3)

  term1 = ss * Sin(az12)
  term2 = cosu1 * cs - sinu1 * ss * Cos(az12)
  tanlambda = term1 / term2
  lambda = Atan2(term1, term2)

  c = flat / 16# * Square(cosalpha) * (4# + flat * (4# - 3# * Square(cosalpha)))

  omega = lambda - (1# - c) * flat * sinalpha * _
            (sigma + c * ss * (c2sm + c * cs * (-1# + 2# * Square(c2sm))))

  long2 = long1 + omega

  term1 = -sinu1 * ss + cosu1 * cs * Cos(az12)
  az21 = Atan2(sinalpha, term1)

  lat2 = ModLatitude(lat2)
  long2 = ModLongitude(long2)
  az21 = ModAzimuth(az21) + PI

  gs.Latitude2 = RadToDeg(lat2)
  gs.Longitude2 = RadToDeg(long2)
  gs.Azimuth21 = RadToDeg(az21)
  lat2 = gs.Latitude2
  long2 = gs.Longitude2
  az12 = gs.Azimuth21
  
  VDirect = WICNOERROR

End Function

Public Function Sex2Deg(ByVal x As Double) As Double

  'given an angle expressed in sexigesimal format:
  '   dd.mmsssss
  'this function returns the angle espressed in
  'decimal degrees:
  '   dd.dddd

  Dim dd    As Double
  Dim d     As Double
  Dim fixdd As Double
  Dim ddfixdd As Double

  dd = Abs(x)
  fixdd = Fix(dd)
  ddfixdd = dd - fixdd + 0.0000000005     'fudge it
  d = fixdd + Fix(100# * ddfixdd) / 60#
  d = d + (10000# * ddfixdd - 100# * Fix(100# * _
                    ddfixdd)) / 3600#
  Sex2Deg = Sgn(x) * d

End Function

Public Function FmtDMS(ByVal x As Double, ByVal n As Integer, _
                        ByVal m As Integer) As String

  'this Function formats angular and time quantities.
  ' Input:  x decimal value - function restricts number
  '                           of decimals to 5
  '        n number of seconds decimal digits
  '        m = 1 convert decimal degrees to:
  '            ddd mm ss      if n = 0
  '            ddd mm ss.f    if n = 1
  '            ddd mm ss.ff   if n = 2
  '            ddd mm ss.fff  if n = 3
  '            ddd mm ss.ffff if n = 4
  '            ddd mm ss.fffff if n >= 5
  '        m = 2 converts decimal hours to:
  '            hh mm ss       if n = 0
  '            hh mm ss.f     if n = 1
  '            hh mm ss.ff    if n = 2
  '            hh mm ss.fff   if n = 3
  '            hh mm ss.ffff  if n = 4
  '            hh mm ss.fffff if n >= 5
  'NOTE: this function rounds seconds at the last decimal place
  
  Dim dh(1 To 2) As String
  Dim mm(1 To 2) As String
  Dim ss(1 To 2) As String
  Dim s As String
  Dim degr As Double
  Dim minn As Double
  Dim sec  As Double
  Dim fig  As Double
  Dim absx As Double

  dh(1) = Chr$(176) '----- degree symbol
  mm(1) = Chr$(39)  '----- arc-min symbol
  ss(1) = Chr$(34)  '----- arc-sec symbol
  dh(2) = "h"
  mm(2) = "m"
  ss(2) = "s"

  absx = Abs(x)
  
  'restrict seconds to no more than 5 decimal places
  If n > 5 Then
    n = 5
  End If
  
  '----- determine how many digits before the decimal
  If (absx < 100#) Then
    fig = 2
  Else
    fig = 3
  End If

  degr = Fix(absx)
  absx = 60# * (absx - degr)
  minn = Fix(absx)
  sec = 60# * (absx - minn)
  sec = Round(sec, n)

  'handle seconds and minutes for rollover
  If (sec >= 60#) Then
    sec = 0#
    minn = minn + 1#
  End If

  'handle fix minutes and degrees/hours for rollover
  If (minn >= 60#) Then
    minn = 0#
    degr = degr + 1#
  End If

  If (((degr = 24#) And m = 2#) Or (degr = 360#)) Then
    degr = 0#
  End If

  s = ""
  If x < 0# Then
    s = "-"
  ElseIf m = 1 Then
    s = "+"
  End If

  s = s & Right$(Str$(1000 + degr), fig) + dh(m)
  s = s & " " & Right$(Str$(100 + minn), 2) + mm(m)
  If n = 0 Then
    s = s & " " & Mid$(LTrim$(Str$(100 + sec + 0.000001)), 2, n + 2) + ss(m)
  Else
    s = s & " " & Mid$(LTrim$(Str$(100 + sec + 0.000001)), 2, n + 3) + ss(m)
  End If

  FmtDMS = s

End Function
Public Function FmtDMS1(ByVal x As Double, _
                        ByVal n As Integer, _
                        ByVal m As Integer, _
                        ByVal l As Integer) As String

  'this Function formats angular and time quantities.
  ' Input:  x decimal value - function restricts number
  '                           of decimals to 5
  '         n number of digits to right of decimal
  '         l format of output string
  '           1 = decimal degrees
  '               i.e. ddd.dddd (for n=4)
  '           2 = degrees and decimal minutes
  '               i.e. ddd mm.mmmm (for n=4)
  '           3 = degrees, minutes and decimal seconds
  '               i.e. ddd mm ss.ssss (for n=4)
  '         m = 1 formatting to degrees
  '         m = 2 formatting to hours
  
  'NOTE: this function rounds at the last decimal place
  
  Dim dh(1 To 2) As String
  Dim mm(1 To 2) As String
  Dim ss(1 To 2) As String
  Dim s As String
  Dim fstr As String
  Dim dstr As String
  Dim mstr As String
  Dim sstr As String
  
  Dim degr As Double
  Dim minn As Double
  Dim sec  As Double
  Dim absx As Double
  Dim df As Double
  Dim d As Double
  Dim t As Integer
  
  dh(1) = Chr$(176) '----- degree symbol
  mm(1) = Chr$(39)  '----- arc-min symbol
  ss(1) = Chr$(34)  '----- arc-sec symbol
  dh(2) = "h"
  mm(2) = "m"
  ss(2) = "s"

  absx = Abs(x)
  
  'restrict decimal places to no more than 5
  If n > 5 Then
    n = 5
  End If
  
  degr = Fix(absx)
  absx = 60# * (absx - degr)
  minn = Fix(absx)
  sec = 60# * (absx - minn)
  minn = absx
  sec = Round(sec, n)

  'handle seconds and minutes for rollover
  If (sec >= 60#) Then
    sec = 0#
    minn = minn + 1#
  End If

  'handle fix minutes and degrees/hours for rollover
  If (minn >= 60#) Then
    minn = 0#
    degr = degr + 1#
  End If

  If (((degr = 24#) And m = 2#) Or (degr = 360#)) Then
    degr = 0#
  End If
  
  s = ""
  If x < 0# Then
    s = "-"
  ElseIf m = 1 Then
    s = "+"
  End If
  
  Select Case l
    Case 1:     'decimal degrees
      d = degr + minn / 60# + sec / 3600#
      dstr = LeftDecStr(d)
      fstr = dstr & "." & String(n, "#") 'build format string
      s = s & Format(d, fstr) + dh(m)
    Case 2:     'degrees and decimal minutes
      dstr = LeftDecStr(degr)
      mstr = LeftDecStr(minn + sec / 60#)
      mstr = mstr & "." & String(n, "#")
      s = s & Format(degr, dstr) & dh(m) & " " & _
              Format(minn, mstr) & mm(m)
    Case 3:     ' degrees, minutes and decimal seconds
      dstr = LeftDecStr(degr)
      mstr = LeftDecStr(minn)
      sstr = LeftDecStr(sec)
      sstr = sstr & "." & String(n, "#")
      s = s & Format(degr, dstr) & dh(m) & " " & _
              Format(minn, mstr) & mm(m) & " " & _
              Format(sec, sstr) & ss(m)
  End Select
  
  FmtDMS1 = s

End Function

Public Function LeftDecStr(ByVal d As Double) As String

  'this function is intended to return a Format() compatible
  'string segment that allows the printing of the integr part
  'of d:
  '     d = 0.456   returns "0"
  '     d = 1.456   returns "#"
  '     d = 12.456  returns "##"
  '     d = 123.456 returns "###"
  '     etc.
  
  Dim t As Integer
  Dim df As Double
  Dim s As String
  
  s = ""
  t = 0
  df = Fix(d)
  Do While df > 0
    df = Fix(df / 10)
    t = t + 1
  Loop
  
  If t = 0 Then
    s = "0"
  Else
    s = String(t, "#")
  End If
  
  LeftDecStr = s
  
End Function
                        

Public Function ParseLatLong(ByRef d As Double, s As String) As Integer
'this function parses the string s and extracts the degrees, minutes and
'seconds. The string s must be in the format dd:mm:ss.ss or dd:mm.mm or dd.ddd The
'string delimiters can be a colon (:) or a semicolon (;) or a comma (,).
'The angle is returned as decimal degrees in d.

'The function returns:
' WICNOERROR if OK
' WICBADTOKENCT if there are too many tokens
' WICNOTNUMERIC if any token is not numeric
' WICBADMINSEC if either minutes exceeds WICMAXMINS or seconds exceed WICMAXSECS

  Const delims As String = ":;,"
  
  Dim tstr As String
  Dim tokenCount As Long
  Dim maxTokens As Integer
  Dim i As Integer
  Dim slen As Integer
  Dim Token() As String
  Dim dms As DMSStruct
  
  'initialize
  dms.Degrees = 0#
  dms.Minutes = 0#
  dms.Seconds = 0#
  d = 0#
  dms.Sign = WICPOSSIGN
  tokenCount = 0
  tstr = ""
  
  tokenCount = Tokenizer(s, _
                         Token(), _
                         delims, _
                         False)
                         
  If tokenCount < 0 Or tokenCount > WICTOKENCT - 1 Then
    ParseLatLong = WICBADTOKENCT
    Exit Function
  End If
  
  'convert tokens to numeric if possible
  For i = 1 To tokenCount + 1
    tstr = LTrim(RTrim(Token(i - 1))) 'get next token
      If IsNumeric(tstr) Then
        Select Case i
          Case 1:
            dms.Degrees = Val(tstr)
            'check for negative
            If dms.Degrees < 0# Then
              dms.Degrees = Abs(dms.Degrees)
              dms.Sign = WICNEGSIGN
            End If
          Case 2:
            dms.Minutes = Abs(Val(tstr))
          Case 3:
            dms.Seconds = Abs(Val(tstr))
        End Select
      Else
        ParseLatLong = WICNOTNUMERIC
        Exit Function
      End If
  Next i
  'now check minutes and seconds.
  If dms.Minutes > WICMAXMINS Or dms.Seconds > WICMAXSECS Then
    ParseLatLong = WICBADMINSEC
    Exit Function
  End If
  
  d = DMStoD(dms)   'return the value in decimal degrees
  
  ParseLatLong = WICNOERROR
  
End Function



